/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.eingang.common.vorgang;

import static org.assertj.core.api.Assertions.*;
import static org.junit.jupiter.api.Assertions.*;
import static org.mockito.Mockito.*;

import java.time.LocalDate;

import org.apache.commons.lang3.RandomStringUtils;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.params.ParameterizedTest;
import org.junit.jupiter.params.provider.ValueSource;
import org.mockito.InjectMocks;
import org.mockito.Spy;

import de.ozgcloud.common.errorhandling.TechnicalException;

class VorgangNummerSupplierTest {

	@Spy
	@InjectMocks
	private VorgangNummerSupplier vorgangNummerSupplier;


	@Test
	@DisplayName("should add random suffix of length 6")
	void shouldAddSuffix() {
		var result = vorgangNummerSupplier.get();

		assertThat(getSuffix(result)).hasSize(VorgangNummerSupplier.SUFFIX_LENGTH);
	}

	@Test
	void shouldHaveSuffixSize() {
		var result = vorgangNummerSupplier.get(3);

		assertThat(getSuffix(result)).hasSize(3);
	}

	@DisplayName("should throw exception when")
	@ParameterizedTest(name = "suffix length {0}")
	@ValueSource(ints = { -1, 0 })
	void shouldThrowException(int suffixLength) {
		assertThrows(TechnicalException.class, () -> vorgangNummerSupplier.get(suffixLength));
	}

	private String getSuffix(String string) {
		return string.substring(string.indexOf('-') + 1);
	}

	@Test
	void shouldCallGetRandomString() {
		try (var randomStringUtils = mockStatic(RandomStringUtils.class)) {
			vorgangNummerSupplier.get();

			randomStringUtils.verify(() -> RandomStringUtils.random(VorgangNummerSupplier.SUFFIX_LENGTH, VorgangNummerSupplier.BASE30_ALPHABET));
		}
	}

	@Test
	void shouldHaveSize() {
		var result = vorgangNummerSupplier.get();

		assertThat(getPrefix(result)).hasSize(5);
	}

	private String getPrefix(String string) {
		return string.substring(0, string.indexOf('-') + 1);
	}

	@Test
	void shouldAddLastYearNumberFirst() {
		var lastYearNumber = "" + LocalDate.now().getYear() % 10;

		var result = vorgangNummerSupplier.get();

		assertThat(result.substring(0, 1)).isEqualTo(lastYearNumber);
	}

	@Test
	void shouldAddMonthValueSecond() {
		var monthHexValue = "%X".formatted(LocalDate.now().getMonthValue());

		var result = vorgangNummerSupplier.get();

		assertThat(result.substring(1, 2)).isEqualTo(monthHexValue);
	}

	@Test
	void shouldAddDayValueThird() {
		var dayValue = "%02d".formatted(LocalDate.now().getDayOfMonth());

		var result = vorgangNummerSupplier.get();

		assertThat(result.substring(2, 4)).isEqualTo(dayValue);
	}

	@Test
	void shouldAddHyphenAtEnd() {
		var result = vorgangNummerSupplier.get();

		assertThat(result.charAt(4)).isEqualTo('-');
	}

}