/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.eingang.formcycle.common.errorhandling;

import java.util.UUID;

import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.ControllerAdvice;
import org.springframework.web.bind.annotation.ExceptionHandler;
import org.springframework.web.context.request.WebRequest;
import org.springframework.web.servlet.mvc.method.annotation.ResponseEntityExceptionHandler;

import de.ozgcloud.common.errorhandling.ExceptionUtil;
import de.ozgcloud.common.errorhandling.TechnicalException;
import io.grpc.StatusRuntimeException;
import lombok.extern.log4j.Log4j2;

@ControllerAdvice
@Log4j2
public class FormcycleExceptionHandler extends ResponseEntityExceptionHandler {

	static final String TECHNICAL_EXCEPTION_MESSAGE = "Cannot process request.";
	static final String CREATE_VORGANG_EXCEPTION_MESSAGE = "Cannot create vorgang.";
	static final String UNEXPECTED_EXCEPTION_MESSAGE = "An unexpected error occurred.";

	private static final String EXCEPTION_ID_TEMPLATE = "(ExceptionId:";

	@ExceptionHandler({ TechnicalException.class })
	public ResponseEntity<InternalExceptionDto> handleTechnicalException(TechnicalException e, WebRequest request) {
		LOG.error(TECHNICAL_EXCEPTION_MESSAGE, e);
		return buildResponseEntity(TECHNICAL_EXCEPTION_MESSAGE, e.getExceptionId());
	}

	@ExceptionHandler({ StatusRuntimeException.class })
	public ResponseEntity<InternalExceptionDto> handleStatusRuntimeException(StatusRuntimeException e, WebRequest request) {
		var logMessage = TECHNICAL_EXCEPTION_MESSAGE;
		var exceptionId = getExceptionId(e.getMessage());
		if (!hasExceptionId(e.getMessage())) {
			logMessage = ExceptionUtil.formatMessageWithExceptionId(CREATE_VORGANG_EXCEPTION_MESSAGE, exceptionId);
		}
		LOG.error(logMessage, e);
		return buildResponseEntity(CREATE_VORGANG_EXCEPTION_MESSAGE, exceptionId);
	}

	boolean hasExceptionId(String message) {
		return message.contains(EXCEPTION_ID_TEMPLATE);
	}

	String getExceptionId(String message) {
		try {
			return message.substring(message.indexOf(EXCEPTION_ID_TEMPLATE) + 14, message.indexOf(")"));
		} catch (IndexOutOfBoundsException e) {
			return createExceptionId();
		}
	}

	@ExceptionHandler({ RuntimeException.class })
	public ResponseEntity<InternalExceptionDto> handleUnexpectedException(RuntimeException e, WebRequest request) {
		var exceptionId = createExceptionId();
		var messageWithExceptionId = ExceptionUtil.formatMessageWithExceptionId(UNEXPECTED_EXCEPTION_MESSAGE, exceptionId);
		LOG.error(messageWithExceptionId, e);
		return buildResponseEntity(UNEXPECTED_EXCEPTION_MESSAGE, exceptionId);
	}

	String createExceptionId() {
		return UUID.randomUUID().toString();
	}

	ResponseEntity<InternalExceptionDto> buildResponseEntity(String message, String exceptionId) {
		return ResponseEntity.internalServerError().body(buildInternalExceptionDto(message, exceptionId));
	}

	InternalExceptionDto buildInternalExceptionDto(String message, String exceptionId) {
		return InternalExceptionDto.builder().message(message).exceptionId(exceptionId).build();
	}
}
