/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.eingang.formsolutions;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.util.Collections;
import java.util.HashMap;
import java.util.Map;
import java.util.Objects;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.stereotype.Component;

import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;

import de.ozgcloud.common.errorhandling.TechnicalException;
import de.ozgcloud.eingang.common.formdata.FormData;
import de.ozgcloud.eingang.common.formdata.IncomingFile;

@Component
class FormSolutionsRequestMapper {
	static final TypeReference<Map<String, Object>> VALUE_TYPE_REF = new TypeReference<Map<String, Object>>() {
	};

	private static final String FILE_NAME_JSON_REPRESENTATION = "form-data.json";
	static final String FILE_NAME_PDF_REPRESENTATION = "eingang.pdf";

	static final String FORMDATA_FIELD_ZUSTAENDIGE_STELLE = "zustaendigeStelle";
	public static final String FORMDATA_FIELD_ASSISTANT = "assistant";
	public static final String FORMDATA_FIELD_POSTKORBHANDLE = "postkorbhandle";
	static final String FORMDATA_FIELD_TRANSACTION_ID = "transactionId";

	@Autowired
	private FormSolutionsAttachmentsMapper attachmentMapper;
	@Autowired
	private ObjectMapper objectMapper;

	public FormData map(File jsonFile) {
		var eingang = mapEingang(jsonFile);

		return buildFormData(jsonFile, eingang);
	}

	FormData buildFormData(File jsonFile, FormSolutionsEingang eingang) {
		var builder = FormData.builder()
				.formData(buildFormDataMap(eingang))
				.attachments(attachmentMapper.mapAttachments(eingang.getZip()))
				.representation(buildJsonFile(jsonFile));
		var numberOfRepresentations = 1;

		if (Objects.nonNull(eingang.getPdf())) {
			builder.representation(buildPdfFile(eingang.getPdf()));
			numberOfRepresentations++;
		}

		return builder.numberOfRepresentations(numberOfRepresentations).build();
	}

	Map<String, Object> buildFormDataMap(FormSolutionsEingang eingang) {
		Map<String, Object> map = new HashMap<>();
		addIfValueNotNull(map, FORMDATA_FIELD_ASSISTANT, eingang.getAssistant());
		addIfValueNotNull(map, FORMDATA_FIELD_POSTKORBHANDLE, eingang.getPostkorbhandle());
		addIfValueNotNull(map, FORMDATA_FIELD_TRANSACTION_ID, eingang.getTransactionId());
		addIfValueNotNull(map, FORMDATA_FIELD_ZUSTAENDIGE_STELLE, eingang.getZustaendigeStelle());

		return Collections.unmodifiableMap(map);
	}

	private Map<String, Object> addIfValueNotNull(Map<String, Object> map, String key, Object value) {
		if (Objects.nonNull(value)) {
			map.put(key, value);
		}
		return map;
	}

	FormSolutionsEingang mapEingang(File jsonFile) {
		try (var in = new FileInputStream(jsonFile)) {
			return objectMapper.readValue(in, FormSolutionsEingang.class);
		} catch (IOException e) {
			throw new TechnicalException("Error parsing JSON from FormSolutions-Server", e);
		}
	}

	private IncomingFile buildJsonFile(File jsonFile) {
		return IncomingFile.builder()
				.file(jsonFile)
				.contentType(MediaType.APPLICATION_JSON_VALUE)
				.name(FILE_NAME_JSON_REPRESENTATION)
				.size(jsonFile.length())
				.build();
	}

	private IncomingFile buildPdfFile(File pdfFile) {
		return IncomingFile.builder()
				.file(pdfFile)
				.contentType(MediaType.APPLICATION_PDF_VALUE)
				.name(FILE_NAME_PDF_REPRESENTATION)
				.size(pdfFile.length())
				.build();
	}
}