/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.eingang.formsolutions;

import static de.ozgcloud.eingang.common.formdata.FormSolutionsTestFactory.*;
import static org.assertj.core.api.Assertions.*;

import java.io.InputStream;
import java.util.List;
import java.util.Map;

import org.assertj.core.api.ObjectAssert;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.boot.test.mock.mockito.SpyBean;
import org.springframework.test.context.ActiveProfiles;

import de.ozgcloud.common.binaryfile.TempFileUtils;
import de.ozgcloud.common.test.TestUtils;
import de.ozgcloud.eingang.common.formdata.FormData;
import de.ozgcloud.eingang.common.formdata.IncomingFile;
import de.ozgcloud.eingang.common.formdata.IncomingFileGroup;
import de.ozgcloud.eingang.semantik.enginebased.formsolutions.FormSolutionsEngineBasedAdapter;
import lombok.SneakyThrows;

@SpringBootTest
@ActiveProfiles({ "local", "itcase" })
class FormSolutionsRequestMapperITCase {

	private static final String COMPONENTS = "components";
	private static final String PANELS = "panels";

	@SpyBean
	private FormSolutionsRequestMapper mapper;

	@Nested
	class TestParseExampleData {
		@Test
		void shouldParseData() {
			var parsed = parseRequestData(SIMPLE_JSON_DATA);

			assertThat(parsed).isNotNull();
		}

		@Test
		void shouldContainRawData() {
			var parsed = parseRequestData(SIMPLE_JSON_DATA);

			assertThat(parsed.getFormData()).isNotNull();
		}

		@Nested
		class TestWithPanels {
			@Test
			void shouldContainPanel() {
				var parsed = parseAndGetPanel();

				assertThat(parsed).isNotNull();
			}

			@Test
			void shouldHaveIdentifier() {
				var panel = parseAndGetPanel();

				assertThat(panel.get(FormSolutionsEngineBasedAdapter.IDENTIFIER_KEY)).isNotNull();
			}

			@Test
			@SuppressWarnings("unchecked")
			void shouldHaveComponents() {
				var panel = parseAndGetPanel();
				var components = (List<Map<String, Object>>) panel.get(COMPONENTS);

				assertThat(components).hasSize(2);
			}
		}

		private Map<String, Object> parseAndGetPanel() {
			return FormSolutionsRequestMapperITCase.this.parseAndGetPanel(SIMPLE_JSON_DATA);
		}
	}

	@Nested
	class TestParseNestedComponents {

		@Test
		void shouldParseData() {
			var parsed = parseRequestData(NESTED_COMPONENTS_JSON);

			assertThat(parsed).isNotNull();
		}

		@Test
		@SuppressWarnings("unchecked")
		void shouldHaveComponents() {
			var panel = parseAndGetPanel();

			assertThat((List<Map<String, Object>>) panel.get(COMPONENTS)).hasSize(1);
		}

		@Test
		void shouldHaveIdentifier() {
			var component = parseAndGetComponent();

			assertThat(component).containsEntry(FormSolutionsEngineBasedAdapter.IDENTIFIER_KEY, OBJEKTGRUPPE_0);
		}

		@Test
		void shouldHaveNestedComponents() {
			var component = parseAndGetComponent();

			assertThat(component.keySet()).hasSize(3);
		}

		@SuppressWarnings("unchecked")
		private Map<String, Object> parseAndGetComponent() {
			return ((List<Map<String, Object>>) parseAndGetPanel().get(COMPONENTS)).get(0);
		}

		private Map<String, Object> parseAndGetPanel() {
			return FormSolutionsRequestMapperITCase.this.parseAndGetPanel(NESTED_COMPONENTS_JSON);
		}

	}

	@Nested
	class TestParsePdfRepresentation {

		@Test
		void shouldHaveRepresentations() {
			var parsed = parseRequestData(PDF_REPRESENTATION_JSON);

			assertThat(parsed.getRepresentations()).hasSize(2);
		}

		@Test
		@SneakyThrows
		void shouldHavePdf() {
			var parsed = parseRequestData(PDF_REPRESENTATION_JSON);

			ObjectAssert<IncomingFile> firstRepresentationAssert = assertThat(parsed.getRepresentations())
					.filteredOn(inFile -> inFile.getContentType().equals("application/pdf")).singleElement();
			firstRepresentationAssert.extracting(IncomingFile::getName).isEqualTo("eingang.pdf");
			firstRepresentationAssert.extracting(IncomingFile::getContentStream).extracting(stream -> toArray(stream))
					.isEqualTo(PDF_VALUE_DECODED.getBytes());
		}
	}

	@Nested
	class TestParseAttachmentZip {

		@Test
		@SneakyThrows
		void shouldHaveZip() {
			var parsed = parseRequestData(ZIP_ATTACHMENT_JSON);

			ObjectAssert<IncomingFileGroup> firstAttachmentAssert = assertThat(parsed.getAttachments()).hasSize(1).first();
			firstAttachmentAssert.extracting(IncomingFileGroup::getName).isEqualTo(FormSolutionsAttachmentsMapper.FILE_GROUP_ZIP_NAME);
			var attachmentFileAssert = firstAttachmentAssert.extracting(fileGroup -> fileGroup.getFiles().get(0));
			attachmentFileAssert.extracting(IncomingFile::getName).isEqualTo("attachments.zip");
			attachmentFileAssert.extracting(file -> toArray(file.getContentStream())).isEqualTo(ZIP_VALUE_DECODED.getBytes());
		}

	}

	// TODO remove this method when TestUtils is not throwing Exception anymore.
	@SneakyThrows
	private byte[] toArray(InputStream stream) {
		return TestUtils.contentStreamToByteArray(stream);
	}

	@SuppressWarnings("unchecked")
	private Map<String, Object> parseAndGetPanel(String json) {
		var data = (Map<String, Object>) parseRequestData(json).getFormData().get(FormSolutionsEngineBasedAdapter.ASSISTANT);
		return ((List<Map<String, Object>>) data.get(PANELS)).get(0);
	}

	private FormData parseRequestData(String json) {
		var file = TempFileUtils.writeTmpFile(json);
		var result = mapper.map(file);
		file.delete();
		return result;
	}
}
