/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.eingang.formsolutions;

import static de.ozgcloud.eingang.common.formdata.FormSolutionsTestFactory.*;
import static de.ozgcloud.eingang.formsolutions.FormSolutionsRequestMapper.*;
import static de.ozgcloud.eingang.semantik.enginebased.formsolutions.FormSolutionsEngineBasedAdapter.*;
import static org.assertj.core.api.Assertions.*;
import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;

import java.io.File;
import java.io.InputStream;
import java.util.Collections;
import java.util.List;
import java.util.Map;

import org.junit.jupiter.api.AfterEach;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.ArgumentCaptor;
import org.mockito.Captor;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.JsonMappingException;
import com.fasterxml.jackson.databind.ObjectMapper;

import de.ozgcloud.common.binaryfile.TempFileUtils;
import de.ozgcloud.common.errorhandling.TechnicalException;
import de.ozgcloud.eingang.common.formdata.IncomingFileGroup;
import de.ozgcloud.eingang.common.formdata.IncomingFileGroupTestFactory;
import lombok.SneakyThrows;

class FormSolutionsRequestMapperTest {

	private static final String COMPONENTS = "components";
	private static final String STRING_VALUE = "stringValue";
	private static final String PANELS = "panels";

	@Spy
	@InjectMocks
	private FormSolutionsRequestMapper mapper;

	@Mock
	private FormSolutionsAttachmentsMapper attachmentMapper;

	@Spy
	private ObjectMapper objectMapper = new ObjectMapper();

	private File simpleJsonFile;
	private File nestedComponenetJsonFile;

	@BeforeEach
	void writeJsonFile() {
		simpleJsonFile = TempFileUtils.writeTmpFile(SIMPLE_JSON_DATA);
		nestedComponenetJsonFile = TempFileUtils.writeTmpFile(NESTED_COMPONENTS_JSON);
	}

	@AfterEach
	void delJsonFile() {
		simpleJsonFile.delete();
		nestedComponenetJsonFile.delete();
	}

	@Nested
	class TestJsonToEingangMapping {

		@Test
		void shouldMapControlValues() {
			var eingang = mapper.mapEingang(simpleJsonFile);

			assertThat(eingang).isNotNull().usingRecursiveComparison()
					.ignoringFields("zip", "pdf", "assistant").isEqualTo(FormSolutionsEingangTestFactory.create());
		}

		@Test
		void shouldHaveAssistantData() {
			var eingang = mapper.mapEingang(simpleJsonFile);

			assertThat(eingang.getAssistant()).isNotEmpty();
		}

		@Test
		@SneakyThrows
		void shouldHandleJsonException() throws JsonMappingException, JsonProcessingException {
			doThrow(JsonProcessingException.class).when(objectMapper).readValue(any(InputStream.class), eq(FormSolutionsEingang.class));

			assertThatThrownBy(() -> mapper.mapEingang(simpleJsonFile)).isInstanceOf(TechnicalException.class);
		}

		@Test
		void shouldContainFormIdentifier() {
			var eingang = mapper.mapEingang(simpleJsonFile);

			assertThat(eingang.getAssistant()).containsEntry(IDENTIFIER_KEY, IDENTIFIER_VALUE);
		}

		@Nested
		class TestPanels {
			@Test
			void shouldContainPanels() {
				var eingang = mapper.mapEingang(simpleJsonFile);

				assertThat(getPanels(eingang)).isNotNull();
			}

			@Test
			void shouldContainPanelIdentifier() {
				var eingang = mapper.mapEingang(simpleJsonFile);

				assertThat(getPanels(eingang).get(0)).containsEntry(IDENTIFIER_KEY, PANEL_ID);
			}

			@Test
			void shouldContainPanelComponets() {
				var eingang = mapper.mapEingang(simpleJsonFile);

				assertThat(getPanels(eingang).get(0).get(COMPONENTS)).isNotNull();
			}

			@Test
			void shouldContainTextComponets() {
				var eingang = mapper.mapEingang(simpleJsonFile);

				assertThat(getComponents(eingang).get(0))
						.containsEntry(IDENTIFIER_KEY, COMPONENT_ID)
						.containsEntry(STRING_VALUE, COMPONENT_VALUE);
			}

			@Test
			void shouldContainDateComponets() {
				var eingang = mapper.mapEingang(simpleJsonFile);

				assertThat(getComponents(eingang).get(1))
						.containsEntry(IDENTIFIER_KEY, DATE_COMPONENT_ID)
						.containsEntry(STRING_VALUE, DATE_COMPONENT_VALUE);
			}

			@Nested
			class TestNestedPanels {
				@Test
				void shouldContainGroup() {
					var eingang = mapper.mapEingang(nestedComponenetJsonFile);

					assertThat(getComponents(eingang).get(0)).containsEntry(IDENTIFIER_KEY, OBJEKTGRUPPE_0);
				}

				@Test
				void shouldContainDateField() {
					var eingang = mapper.mapEingang(nestedComponenetJsonFile);

					assertThat(getNestedComponents(eingang).get(0))
							.containsEntry(IDENTIFIER_KEY, DATE_COMPONENT_ID)
							.containsEntry(STRING_VALUE, DATE_COMPONENT_VALUE);
				}
			}
		}

		@SuppressWarnings("unchecked")
		private List<Map<String, Object>> getComponents(FormSolutionsEingang eingang) {
			return (List<Map<String, Object>>) getPanels(eingang).get(0).get(COMPONENTS);
		}

		@SuppressWarnings("unchecked")
		private List<Map<String, Object>> getNestedComponents(FormSolutionsEingang eingang) {
			return (List<Map<String, Object>>) ((List<Map<String, Object>>) getPanels(eingang).get(0).get(COMPONENTS)).get(0).get(COMPONENTS);
		}

		@SuppressWarnings("unchecked")
		private List<Map<String, Object>> getPanels(FormSolutionsEingang eingang) {
			return (List<Map<String, Object>>) eingang.getAssistant().getOrDefault(PANELS, Collections.emptyList());
		}
	}

	@Nested
	class TestFileMapping {

		@Nested
		class TestMapFiles {

			private File jsonFile;
			@Captor
			private ArgumentCaptor<File> fileCaptor;

			@BeforeEach
			void writeJsonFile() {
				jsonFile = TempFileUtils.writeTmpFile(ATTACHMENTS_JSON);
			}

			@AfterEach
			void deleteTempFile() {
				jsonFile.delete();
			}

			@Test
			void shouldMapZipFile() {
				var eingang = mapper.mapEingang(jsonFile);

				assertThat(eingang.getZip()).exists().content().isEqualTo(ZIP_VALUE_DECODED);
			}

			@Test
			void shouldCallAttachmentMappers() {
				mapper.map(jsonFile);

				verify(attachmentMapper).mapAttachments(fileCaptor.capture());
				assertThat(fileCaptor.getValue()).content().isEqualTo(ZIP_VALUE_DECODED);
			}

			@DisplayName("result should have mapped files field")
			@Test
			void shouldHaveMappedFilesField() {
				IncomingFileGroup fileGroup = IncomingFileGroupTestFactory.create();
				when(attachmentMapper.mapAttachments(any())).thenReturn(List.of(fileGroup));

				var result = mapper.map(jsonFile);

				assertThat(result.getAttachments()).containsExactly(fileGroup);
			}

			@Test
			void shouldAddRepresentations() {
				var result = mapper.map(jsonFile);

				assertThat(result.getRepresentations()).hasSize(2);
			}

			@Test
			void shouldAddNumberOfRepresentations() {
				var result = mapper.map(jsonFile);

				assertThat(result.getNumberOfRepresentations()).isEqualTo(2);
			}
		}

	}

	@Nested
	class TestBuildFormDataMap {

		@Test
		void shouldHavePostkorbHandle() {
			var formData = mapper.buildFormDataMap(FormSolutionsEingangTestFactory.create());

			assertThat(formData).containsEntry(FORMDATA_FIELD_POSTKORBHANDLE, POSTFACH_ID_STELLE);
		}

		@Test
		void shouldHaveZustaendigeStelle() {
			var formData = mapper.buildFormDataMap(FormSolutionsEingangTestFactory.create());

			assertThat(formData).containsKey(FORMDATA_FIELD_ZUSTAENDIGE_STELLE);
		}

		@Test
		void shouldHaveTransactionId() {
			var formData = mapper.buildFormDataMap(FormSolutionsEingangTestFactory.create());

			assertThat(formData).containsKey(FORMDATA_FIELD_TRANSACTION_ID);
		}
	}

}
