/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.eingang.formsolutions;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;

import java.util.List;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.ArgumentCaptor;
import org.mockito.Captor;
import org.mockito.Mock;
import org.mockito.verification.Timeout;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.boot.test.mock.mockito.MockBean;
import org.springframework.test.annotation.DirtiesContext;
import org.springframework.test.context.ActiveProfiles;
import org.springframework.test.util.ReflectionTestUtils;

import de.ozgcloud.common.test.TestUtils;
import de.ozgcloud.eingang.router.VorgangManagerServerResolver;
import de.ozgcloud.vorgang.grpc.binaryFile.BinaryFileServiceGrpc.BinaryFileServiceStub;
import de.ozgcloud.vorgang.grpc.binaryFile.GrpcUploadBinaryFileRequest;
import de.ozgcloud.vorgang.grpc.binaryFile.GrpcUploadBinaryFileResponse;
import de.ozgcloud.vorgang.vorgang.GrpcCreateVorgangRequest;
import de.ozgcloud.vorgang.vorgang.GrpcCreateVorgangResponse;
import de.ozgcloud.vorgang.vorgang.VorgangServiceGrpc.VorgangServiceBlockingStub;
import io.grpc.Channel;
import io.grpc.stub.CallStreamObserver;
import io.grpc.stub.ClientResponseObserver;

@SpringBootTest
@DirtiesContext
@ActiveProfiles({ "local", "itcase" })
public class FormsolutionsITCase {

	@Autowired
	private SendFormEndpoint endpoint;

	@MockBean
	private VorgangManagerServerResolver resolver;

	@Mock
	private Request request;

	@Mock
	private VorgangServiceBlockingStub blockingStub;
	@Mock
	private BinaryFileServiceStub fileStub;
	@Mock
	private CallStreamObserver<GrpcUploadBinaryFileRequest> fileStreamObserver;

	@BeforeEach
	void initVorgangManagerResolver() {
		when(resolver.resolveVorgangServiceBlockingStubByOrganisationseinheitenId(any())).thenReturn(blockingStub);
		when(resolver.resolveBinaryFileServiceStubByOrganisationsEinheitId(any())).thenReturn(fileStub);
		
		Channel mockChannel = mock(Channel.class);
		when(blockingStub.getChannel()).thenReturn(mockChannel );
		when(blockingStub.startCreation(any())).thenReturn(GrpcCreateVorgangResponse.newBuilder().setVorgangId("42").build());

		when(fileStub.uploadBinaryFileAsStream(any())).thenReturn(fileStreamObserver);
	}

	@Nested
	class TestSendingVorgangBasicInformation {

		@Captor
		private ArgumentCaptor<GrpcCreateVorgangRequest> createVorgangRequestCaptor;

		@BeforeEach
		void init() {
			when(request.getJSON()).thenReturn(loadTextFile("SimpleJsonWithAttachments.json"));
		}

		@Test
		void shouldContainZustaendigeStelle() {
			new Thread(() -> endpoint.receiveForm(request)).start();

			var request = getCreateVorgangRequest();

			assertThat(request.getEingang().getZustaendigeStelle().getOrganisationseinheitenId()).isEqualTo("5678");
		}

		private GrpcCreateVorgangRequest getCreateVorgangRequest() {
			verify(blockingStub, timeout(1000)).startCreation(createVorgangRequestCaptor.capture());
			return createVorgangRequestCaptor.getValue();
		}
	}

	@Nested
	class TestReceiveFormWithAttachments {

		@Captor
		private ArgumentCaptor<ClientResponseObserver<GrpcUploadBinaryFileRequest, GrpcUploadBinaryFileResponse>> observerCaptor;

		@Captor
		private ArgumentCaptor<GrpcUploadBinaryFileRequest> requestCaptor;

		@BeforeEach
		void init() {
			when(request.getJSON()).thenReturn(loadTextFile("SimpleJsonWithAttachments.json"));
		}

		@Test
		void shouldSendContentOfAttachment() {
			new Thread(() -> endpoint.receiveForm(request)).start();

			var requests = getFileRequests();

			var fileContent = requests.get(1).getFileContent();
			assertThat(fileContent.isEmpty()).isFalse();
		}

		@Test
		void shouldHaveContentType() {
			new Thread(() -> endpoint.receiveForm(request)).start();

			var requests = getFileRequests();

			var contentType = requests.get(0).getMetadata().getContentType();
			assertThat(contentType).isEqualTo("application/pdf");
		}

		@Test
		void shouldHaveFileSize() {
			new Thread(() -> endpoint.receiveForm(request)).start();

			var requests = getFileRequests();

			var size = requests.get(0).getMetadata().getSize();
			assertThat(size).isEqualTo(6788);
		}

		private List<GrpcUploadBinaryFileRequest> getFileRequests() {
			verify(fileStub, timeout(1000)).uploadBinaryFileAsStream(observerCaptor.capture());
			var onReadyHandler = (Runnable) ReflectionTestUtils.getField(observerCaptor.getValue(), "onReadyHandler");
			onReadyHandler.run();

			verify(fileStreamObserver, new Timeout(2000, times(2))).onNext(requestCaptor.capture());
			var requests = requestCaptor.getAllValues();
			assertThat(requests).isNotEmpty();
			return requests;
		}

	}

	private static String loadTextFile(final String fileName) {
		return TestUtils.loadTextFile(fileName);
	}

}
