/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.eingang.formsolutions;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;

import java.nio.file.Files;
import java.util.List;
import java.util.Optional;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.ArgumentCaptor;
import org.mockito.Captor;
import org.mockito.Mock;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.boot.test.mock.mockito.MockBean;
import org.springframework.boot.test.mock.mockito.SpyBean;
import org.springframework.context.ApplicationContext;
import org.springframework.core.io.Resource;
import org.springframework.test.context.ActiveProfiles;

import de.ozgcloud.eingang.common.formdata.FormData;
import de.ozgcloud.eingang.router.VorgangRemoteService;
import de.ozgcloud.vorgang.common.GrpcObject;
import de.ozgcloud.vorgang.vorgang.GrpcEingang;
import de.ozgcloud.vorgang.vorgang.GrpcPostfachAddress;
import lombok.SneakyThrows;

@ActiveProfiles({ "local", "itcase" })
@SpringBootTest
class SendFormEndpointITCase {

	private final static String FILE_BASE_PATH = "classpath:formular/";

	private final static String FORMULAR_JSON = "RequestJsonContent.json";

	@Autowired
	private ApplicationContext applicationContext;

	@SpyBean
	private SendFormEndpoint endpoint;
	@MockBean
	private VorgangRemoteService vorgangRemoteService;

	@Captor
	private ArgumentCaptor<FormData> formDataCaptor;
	@Captor
	private ArgumentCaptor<GrpcEingang> grpcEingangCaptor;
	@Captor
	private ArgumentCaptor<Optional<String>> organisationsEinheitIdCaptor;

	@DisplayName("Receive form")
	@Nested
	class TestReceiveForm {

		@Mock
		private Request request;

		@BeforeEach
		void mockRequest() {
			when(request.getJSON()).thenReturn(loadJsonContent());
		}

		@SneakyThrows
		private String loadJsonContent() {
			var resource = getResource(FORMULAR_JSON);
			return Files.readString(resource.getFile().toPath());
		}

		@Test
		void shouldCallRemoteService() {
			callEndpoint();

			verify(vorgangRemoteService).createVorgang(any(FormData.class), any(GrpcEingang.class), any());
		}

		@DisplayName("service konto")
		@Nested
		class TestServiceKonto {

			@Test
			void shouldExist() {
				callEndpoint();

				assertThat(grpcEingangCaptor.getValue().getHeader().getServiceKonto()).isNotNull();
			}

			@Test
			void shouldContainsType() {
				callEndpoint();

				assertThat(grpcEingangCaptor.getValue().getHeader().getServiceKonto().getType()).isEqualTo("OSI");
			}

			@Test
			void shouldContainsPostfachAddress() {
				callEndpoint();

				assertThat(grpcEingangCaptor.getValue().getHeader().getServiceKonto().getPostfachAddressesList()).hasSize(1);
			}

			@Nested
			class TestPostfachAddress {

				@Test
				void shouldContainsType() {
					callEndpoint();

					assertThat(getPostfachAddress().getType()).isEqualTo(1);
				}

				@Test
				void shouldContainsVersion() {
					callEndpoint();

					assertThat(getPostfachAddress().getVersion()).isEqualTo("1.0");
				}

				@Test
				void shouldContainsIdentifier() {
					callEndpoint();

					assertThat(getPostfachAddress().getIdentifier()).isInstanceOf(GrpcObject.class);
					assertThat(getPostfachAddress().getIdentifier().getPropertyList()).hasSize(1);
					assertThat(getPostfachAddress().getIdentifier().getPropertyList().get(0).getName()).isEqualTo("postfachId");
					assertThat(getPostfachAddress().getIdentifier().getPropertyList().get(0).getValue(0))
							.isEqualTo("51522620-03d2-4507-b1f0-08d86920efed");
				}

				private GrpcPostfachAddress getPostfachAddress() {
					List<GrpcPostfachAddress> addresses = grpcEingangCaptor.getValue().getHeader().getServiceKonto().getPostfachAddressesList();
					assertThat(addresses).isNotEmpty();
					return addresses.get(0);
				}
			}
		}

		private void callEndpoint() {
			endpoint.receiveForm(request);

			verify(vorgangRemoteService).createVorgang(formDataCaptor.capture(), grpcEingangCaptor.capture(), organisationsEinheitIdCaptor.capture());
		}

		private Resource getResource(String fileName) {
			return applicationContext.getResource(FILE_BASE_PATH + fileName);
		}
	}
}