/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.eingang.intelliform;

import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;

import de.ozgcloud.eingang.common.errorhandling.TechnicalException;
import lombok.extern.log4j.Log4j2;

@Log4j2
@Component
class XmlToJavaMapsMapper {

	static final String REST_RESPONSE_NAME = "rest_response_name";
	static final String FILE = "file";

	@Autowired
	private JsonService jsonService;

	public Document parseAsW3cDocument(InputStream xmlDatenInputStream) {
		try {
			DocumentBuilderFactory docBuilderFactory = DocumentBuilderFactory.newInstance();

			docBuilderFactory.setFeature("http://apache.org/xml/features/disallow-doctype-decl", true);

			DocumentBuilder docBuilder = docBuilderFactory.newDocumentBuilder();
			Document document = docBuilder.parse(xmlDatenInputStream);

			// optional, but recommended
			// http://stackoverflow.com/questions/13786607/normalization-in-dom-parsing-with-java-how-does-it-work
			document.getDocumentElement().normalize();

			return document;

		} catch (IOException | ParserConfigurationException | SAXException e) {
			throw new TechnicalException("Could not parse XML-Daten.xml file content", e);
		}
	}

	public Map<String, Object> mapXmlToJavaMaps(Node node) {
		return mapChildNodes(node.getFirstChild());
	}

	private Map<String, Object> mapChildNodes(Node node) {

		Map<String, Object> childs = new LinkedHashMap<>();

		NodeList nodeList = node.getChildNodes();

		for (int i = 0; i < nodeList.getLength(); i++) {

			Node currentNode = nodeList.item(i);

			if (isElementNode(currentNode)) {

				if (isSimpleTextNode(currentNode)) {

					addChildElement(childs, currentNode, currentNode.getFirstChild().getTextContent());

				} else {

					if (currentNode.getChildNodes().getLength() > 0) {

						addChildElement(childs, currentNode, mapChildNodes(currentNode));
					}
				}
			}
		}

		return childs;
	}

	private boolean isElementNode(Node node) {
		return node.getNodeType() == Node.ELEMENT_NODE;
	}

	private boolean isSimpleTextNode(Node node) {
		return node.getChildNodes().getLength() == 1 && isTextNode(node.getFirstChild());
	}

	private boolean isTextNode(Node node) {
		return node.getNodeType() == Node.TEXT_NODE;
	}

	private void addChildElement(Map<String, Object> childs, Node currentNode, Object content) {
		if (childs.containsKey(currentNode.getNodeName())) {
			addToExistingChildElement(childs, currentNode, content);
		} else {
			childs.put(currentNode.getNodeName(), getContentValue(currentNode, content));
		}
	}

	@SuppressWarnings({ "unchecked", "rawtypes" })
	private void addToExistingChildElement(Map<String, Object> childs, Node currentNode, Object content) {
		var existingChild = childs.get(currentNode.getNodeName());
		if (existingChild instanceof List list) {
			list.add(content);
		} else {
			var list = new ArrayList<Object>();
			list.add(existingChild);
			list.add(content);
			childs.put(currentNode.getNodeName(), list);
		}
	}

	Object getContentValue(Node currentNode, Object content) {
		if (isFileNode(currentNode)) {
			content = createFileContentMap(currentNode, content);
		} else if (isJsonNode(currentNode)) {
			content = getJsonValue(currentNode, ((String) content).trim());
		}
		return content;
	}

	private boolean isFileNode(Node currentNode) {
		return currentNode.getNodeName().equals(FILE);
	}

	private Map<String, Object> createFileContentMap(Node currentNode, Object content) {
		var attributes = currentNode.getAttributes();

		var contentMap = new HashMap<String, Object>();
		contentMap.put("text", content);// TOASK: text oder name?

		for (int i = 0; i < attributes.getLength(); i++) {
			var attribute = attributes.item(i);
			contentMap.put(attribute.getNodeName(), attribute.getNodeValue());
		}
		return contentMap;
	}

	private boolean isJsonNode(Node currentNode) {
		return isRestResponseName(currentNode);
	}

	private boolean isRestResponseName(Node currentNode) {
		return currentNode.getNodeName().equals(REST_RESPONSE_NAME);
	}

	private Object getJsonValue(Node currentNode, String content) {
		try {
			return jsonService.readAsListMap(content);
		} catch (de.ozgcloud.common.errorhandling.TechnicalException e) {
			LOG.error("Error parsing json content from <" + currentNode.getNodeName() + ">.", e);
			return Collections.emptyList();
		}
	}
}