/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.eingang.intelliform;

import static de.ozgcloud.eingang.common.formdata.IncomingFileGroupTestFactory.*;
import static org.assertj.core.api.Assertions.*;
import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;

import java.io.IOException;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.xml.parsers.ParserConfigurationException;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.Mock;
import org.xml.sax.SAXException;

import de.ozgcloud.eingang.common.formdata.FormData;
import de.ozgcloud.eingang.common.formdata.IncomingFile;
import de.ozgcloud.eingang.semantik.enginebased.FilesMapperHelper;

class SemantikFormDataMapperTest {

	private SemantikFormDataMapper mapper;

	@Mock
	private CustomHeaderReader customHeaderReader;

	@BeforeEach
	void setup() {
		mapper = new SemantikFormDataMapper(new XmlToJavaMapsMapper(), new FormDataIncomingFileMapper(),
				new RepresentationsCalculator(), new DepositRequestIncomingFileMapper(), new AttachmentsContentAdder(), customHeaderReader);
	}

	@Nested
	class TestMapFormData {

		private final Deposit deposit = DepositTestFactory.create();

		@Test
		void shouldMapIdField() throws SAXException, IOException, ParserConfigurationException {
			var formData = mapToFormData(deposit);

			assertThat(formData.getId()).isNotNull();
		}

		@Test
		void shouldRemoveFileReferences() {
			Map<String, Object> formDataMap = new HashMap<>(Map.of(FILE_REF1, Map.of(SemantikFormDataMapper.FILE, Map.of(ID, VENDOR_ID_XXX))));

			mapper.removeMappedFileReferences(FILE_GROUPS, formDataMap);

			assertThat(formDataMap).doesNotContainKey(FILE_REF1);
		}

		@Nested
		class TestMapDepositRepresentations {

			@Test
			void shouldMap() throws SAXException, IOException, ParserConfigurationException {
				var formData = mapToFormData(deposit);

				assertThat(getRepresentations(formData)).hasSize(2);
			}

			@Test
			void shouldContainsXmlData() {
				var formData = mapToFormData(deposit);

				var xmlRepresentation = getRepresentations(formData).stream()
						.filter(xml -> xml.getContentType().equals(AttachmentTestFactory.XML_CONTENT_TYPE)).findFirst();
				assertThat(xmlRepresentation).isPresent();
			}

			@Test
			void shouldContainsPdfData() {
				var formData = mapToFormData(deposit);

				var pdfRepresentation = getRepresentations(formData).stream()
						.filter(pdf -> pdf.getContentType().equals(AttachmentTestFactory.PDF_ATTACHMENT_CONTENT_TYPE)).findFirst();
				assertThat(pdfRepresentation).isPresent();
			}

			@Test
			void shouldContainsPdfAttributes() {
				var formData = mapToFormData(deposit);

				var pdfAttachment = getRepresentations(formData).stream()
						.filter(pdf -> pdf.getContentType().equals(AttachmentTestFactory.PDF_ATTACHMENT_CONTENT_TYPE)).findFirst();

				assertThat(pdfAttachment).isPresent();
				assertThat(pdfAttachment.get().getContentType()).isEqualTo(AttachmentTestFactory.PDF_ATTACHMENT_CONTENT_TYPE);
				assertThat(pdfAttachment.get().getVendorId()).isEqualTo(AttachmentTestFactory.PDF_ATTACHMENT_ID);
				assertThat(pdfAttachment.get().getName()).isEqualTo(AttachmentTestFactory.PDF_ATTACHMENT_NAME);
				assertThat(pdfAttachment.get().getContentStream()).hasBinaryContent(AttachmentTestFactory.PDF_ATTACHMENT_CONTENT);
			}

			@SuppressWarnings("unchecked")
			private List<IncomingFile> getRepresentations(FormData formData) {
				return (List<IncomingFile>) ((Map<String, Object>) formData.getFormData().get(FilesMapperHelper.FIELD_NAME_MAPPED_FILES))
						.get(FilesMapperHelper.REPRESENTATIONS);
			}
		}

		@Nested
		class TestMapHeader {

			@Test
			void shouldMapHeader() {
				var formData = mapToFormData(deposit);

				var formHeader = getHeader(formData);
				assertThat(formHeader)
						.containsEntry(SemantikFormDataMapper.HEADER_ID, "20201118365670866101")
						.containsEntry(SemantikFormDataMapper.HEADER_TIMESTAMP, "2020-11-18T09:09:27.627Z")
						.containsEntry(SemantikFormDataMapper.HEADER_FORM_ID, "waffen/kleinerWaffenschein")
						.containsEntry(SemantikFormDataMapper.HEADER_FORM, "Kleiner Waffenschein gem. § 10 Abs. 4 Satz 4 Waffengesetz (WaffG)")
						.containsEntry(SemantikFormDataMapper.HEADER_SENDER, "afm.schleswig-holstein.de")
						.containsEntry(SemantikFormDataMapper.HEADER_CUSTOMER, "Einheitlicher Ansprechpartner")
						.containsEntry(SemantikFormDataMapper.HEADER_CUSTOMER_ID, "ea-sh")
						.containsEntry(SemantikFormDataMapper.HEADER_CLIENT, "Schleswig-Holstein")
						.containsEntry(SemantikFormDataMapper.HEADER_CLIENT_ID, "land");
			}

			@Test
			void shouldAddBayernHeader() {
				Map<String, Object> bayernHeader = Map.of(CustomHeaderReader.HEADER_POSTFACH_ID, CustomHeaderTestFactory.POSTFACH_ID);
				when(customHeaderReader.getHeader(any())).thenReturn(bayernHeader);

				var formData = mapToFormData(deposit);

				verify(customHeaderReader).getHeader(any());
				assertThat(getHeader(formData)).containsEntry(CustomHeaderReader.HEADER_POSTFACH_ID, CustomHeaderTestFactory.POSTFACH_ID);
			}

			@SuppressWarnings("unchecked")
			private Map<String, Object> getHeader(FormData formData) {
				return (Map<String, Object>) formData.getFormData().get(SemantikFormDataMapper.HEADER_FIELD);
			}
		}

		private FormData mapToFormData(Deposit deposit) {
			return mapper.mapToFormData(deposit);
		}
	}
}