/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.eingang.router;

import static org.assertj.core.api.Assertions.*;

import java.util.HashMap;
import java.util.Map;
import java.util.Optional;

import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;

import de.ozgcloud.eingang.router.GrpcClientsProperties.ClientProperty;
import jakarta.validation.Validation;
import jakarta.validation.Validator;
import jakarta.validation.ValidatorFactory;

class VorgangManagerListPropertiesTest {

	@Nested
	class TestSingleRouting {

		private VorgangManagerListProperties props = VorgangManagerListPropertiesTestFactory.createForSingleRouting();

		@Test
		void shouldBeValid() {
			var violations = getValidator().validate(props);

			assertThat(violations).isEmpty();
		}

		@Test
		void shouldViolateMissingRoutingStrategy() {
			props.setRoutingStrategy(null);

			var violations = getValidator().validate(props);

			assertThat(violations).isNotEmpty();
		}

		@Test
		void shouldViolateMissingTargetVorgangManagerName() {
			props.setTargetVorgangManagerName(Optional.empty());

			var violations = getValidator().validate(props);

			assertThat(violations).isNotEmpty();
		}
	}

	@Nested
	class TestMultiRouting {
		private VorgangManagerListProperties props = VorgangManagerListPropertiesTestFactory.createForMultiRouting();

		@Test
		void shouldBeValid() {
			var violations = getValidator().validate(props);

			assertThat(violations).isEmpty();
		}

		@Test
		void shouldViolateMissingFallbackStrategy() {
			props.setFallbackStrategy(null);

			var violations = getValidator().validate(props);

			assertThat(violations).isNotEmpty();
		}

	}

	@Nested
	class TestFallbackFundstelle {
		private VorgangManagerListProperties props = VorgangManagerListPropertiesTestFactory.createWithFundstelle();

		@Test
		void shouldBeValid() {
			var violations = getValidator().validate(props);

			assertThat(violations).isEmpty();
		}

		@Test
		void shouldViolateMissingFundstelle() {
			props.setFundstelleVorgangManagerName(Optional.empty());

			var violations = getValidator().validate(props);

			assertThat(violations).isNotEmpty();
		}
	}

	@Nested
	class TestClientProperties {
		private VorgangManagerListProperties props = VorgangManagerListPropertiesTestFactory.createForMultiRouting();

		@Test
		void shouldViolateMissingVorgangManager() {
			Map<String, ClientProperty> clientMap = new HashMap<>(props.getClientProperties().get().getClient());
			clientMap.remove("vorgang-manager-kiel");
			props.getClientProperties().get().setClient(clientMap);

			var violations = getValidator().validate(props);

			assertThat(violations).isNotEmpty();
		}
	}

	private Validator getValidator() {
		ValidatorFactory factory = Validation.buildDefaultValidatorFactory();
		return factory.getValidator();
	}
}
