/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.eingang.semantik.enginebased;

import java.util.Collections;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.Optional;

import org.springframework.stereotype.Component;

import de.ozgcloud.eingang.common.formdata.FormData;
import de.ozgcloud.eingang.common.formdata.PostfachAddressIdentifier;
import de.ozgcloud.eingang.common.formdata.ServiceKonto;
import de.ozgcloud.eingang.common.formdata.StringBasedIdentifier;
import de.ozgcloud.eingang.common.formdata.ServiceKonto.PostfachAddress;

@Component
public class ServiceKontoBuildHelper {

	public static final int POSTFACH_ADDRESS_DEFAULT = 1;
	public static final String POSTFACH_TYPE_OSI = "OSI";
	public static final String POSTFACH_TYPE_BAYERN_ID = "BayernID";
	public static final String POSTFACH_VERSION = "1.0";

	public static final String REST_RESPONSE_NAME = "rest_response_name";
	public static final String REST_RESPONSE_NAME_MEMBER_SCOPE = "memberscope";
	public static final String REST_RESPONSE_NAME_MEMBER_SCOPE_MAILBOX_TYPE = "mailboxtype";

	public ServiceKonto buildOsiServiceKonto(String postfachId) {
		return buildDefault(postfachId);
	}

	public ServiceKonto buildOsiServiceKonto(String postfachId, FormData formData) {
		return Optional.ofNullable(getRestResponseNames(formData))
				.filter(names -> !names.isEmpty())
				.map(restResponseNames -> buildWithRestResponseNames(postfachId, restResponseNames))
				.orElseGet(() -> buildDefault(postfachId));
	}

	ServiceKonto buildDefault(String postfachId) {
		return ServiceKonto.builder().type(POSTFACH_TYPE_OSI).postfachAddress(buildPostfachAddress(postfachId)).build();
	}

	@SuppressWarnings("unchecked")
	private List<Map<String, Object>> getRestResponseNames(FormData formData) {
		return Optional.ofNullable(formData.getFormData().get(REST_RESPONSE_NAME))
				.filter(Objects::nonNull)
				.map(List.class::cast)
				.orElse(Collections.emptyList());
	}

	ServiceKonto buildWithRestResponseNames(String postfachId, List<Map<String, Object>> restResponseNames) {
		return ServiceKonto.builder()
				.type(POSTFACH_TYPE_OSI)
				.postfachAddresses(buildPostfachAddresses(buildIdentifier(postfachId), restResponseNames))
				.build();
	}

	List<PostfachAddress> buildPostfachAddresses(PostfachAddressIdentifier identifier, List<Map<String, Object>> restResponseNames) {
		return restResponseNames.stream().map(entry -> buildOsiPostfachV1Address(identifier, entry)).toList();
	}

	PostfachAddress buildOsiPostfachV1Address(PostfachAddressIdentifier identifier, Map<String, Object> restResponseName) {
		return buildOsiPostfachV1Address(identifier, getPostfachAddressType(restResponseName));
	}

	PostfachAddress buildOsiPostfachV1Address(PostfachAddressIdentifier identifier, int postfachAddressType) {
		return PostfachAddress.builder()
				.type(postfachAddressType)
				.version(POSTFACH_VERSION)
				.identifier(identifier)
				.build();
	}

	int getPostfachAddressType(Map<String, Object> restResponseName) {
		return getMailboxType(restResponseName);
	}

	private Integer getMailboxType(Map<String, Object> restResponseName) {
		return (Integer) getMemberScope(restResponseName).get(REST_RESPONSE_NAME_MEMBER_SCOPE_MAILBOX_TYPE);
	}

	@SuppressWarnings("unchecked")
	private Map<String, Object> getMemberScope(Map<String, Object> restResponseName) {
		return ((List<Map<String, Object>>) restResponseName.get(REST_RESPONSE_NAME_MEMBER_SCOPE)).get(0);
	}

	public ServiceKonto buildBayernIdServiceKonto(String postfachId) {
		return ServiceKonto.builder().type(POSTFACH_TYPE_BAYERN_ID).postfachAddress(buildPostfachAddress(postfachId)).build();
	}

	PostfachAddress buildPostfachAddress(String postkorbHandle) {
		return PostfachAddress.builder()
				.type(POSTFACH_ADDRESS_DEFAULT)
				.version(POSTFACH_VERSION)
				.identifier(buildIdentifier(postkorbHandle))
				.build();
	}

	private PostfachAddressIdentifier buildIdentifier(String postfachId) {
		return StringBasedIdentifier.builder().postfachId(postfachId).build();
	}

}