/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.eingang.semantik.enginebased.formsolutions;

import static de.ozgcloud.eingang.semantik.enginebased.formsolutions.FormSolutionsEngineBasedAdapter.*;
import static org.assertj.core.api.Assertions.*;

import java.io.File;
import java.nio.file.Files;
import java.nio.file.Path;
import java.util.Map;
import java.util.Optional;

import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.boot.test.mock.mockito.MockBean;
import org.springframework.boot.test.mock.mockito.SpyBean;
import org.springframework.test.context.ActiveProfiles;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;

import de.ozgcloud.common.binaryfile.TempFileUtils;
import de.ozgcloud.common.test.TestUtils;
import de.ozgcloud.eingang.common.errorhandling.TechnicalException;
import de.ozgcloud.eingang.common.formdata.FormData;
import de.ozgcloud.eingang.common.formdata.FormDataTestFactory;
import de.ozgcloud.eingang.common.formdata.IncomingFileGroupTestFactory;
import de.ozgcloud.eingang.common.formdata.IncomingFileTestFactory;
import de.ozgcloud.eingang.semantik.SemantikAdapter;
import lombok.SneakyThrows;

@SpringBootTest
@ActiveProfiles({ "local", "itcase" })
public class FormSolutionsEngineBasedAdapterITCase {

	private static final String ZIP_CONTENT_TYPE = "application/zip";
	private static final String FILE_NAME_ZIP_ATTACHMENT = "attachment-2files.zip";
	public static final String FILE_NAME_PDF_REP = "eingang.pdf";
	public static final String FILE_NAME_JSON_REP = "form-data.json";

	@MockBean
	private SemantikAdapter semantikAdapter;

	@SpyBean
	private FormSolutionsEngineBasedAdapter engineAdapter;

	private ObjectMapper objectMapper = new ObjectMapper();

	@Nested
	class TestEngineBasedAdapter {

		private FormData formData = prepareTestData();

		@Test
		void shouldMap() {
			var data = engineAdapter.parseFormData(formData);

			assertThat(data).isNotNull();
		}

		@Test
		void shouldRemoveRawData() {
			var data = engineAdapter.parseFormData(formData);

			assertThat(data.getFormData().get(ASSISTANT)).isNull();
		}

		@Nested
		class TestAntragsteller {
			@Test
			void shouldMap() {
				var data = engineAdapter.parseFormData(formData);

				assertThat(data.getAntragsteller()).isNotNull();
			}

			@Test
			void shouldMapVorname() {
				var data = engineAdapter.parseFormData(formData);

				assertThat(data.getAntragsteller().getVorname()).isEqualTo("Susanne");
			}

			@Test
			void shouldMapNachname() {
				var data = engineAdapter.parseFormData(formData);

				assertThat(data.getAntragsteller().getNachname()).isEqualTo("Fischer");
			}
		}

		@Nested
		class TestAttachments {
			@Test
			void shouldBeEmptyForNoAttachments() {
				var data = engineAdapter.parseFormData(FormDataTestFactory.createBuilder().clearAttachments().build());

				assertThat(data.getAttachments()).isEmpty();
			}

			@Test
			void shouldMap() {
				var data = engineAdapter.parseFormData(formData);

				assertThat(data.getAttachments()).isNotEmpty();
			}

			@Test
			void shouldMapAttachment() {
				var data = engineAdapter.parseFormData(formData);

				assertThat(data.getAttachments().get(0)).isNotNull();
				assertThat(data.getNumberOfAttachments()).isEqualTo(2);
			}
		}

		@Nested
		class TestHeader {
			@Test
			void shouldMap() {
				var data = engineAdapter.parseFormData(formData);

				assertThat(data.getHeader()).isNotNull();
			}

			@Test
			void shouldMapRequestId() {
				var data = engineAdapter.parseFormData(formData);

				assertThat(data.getHeader().getRequestId()).isEqualTo("KFAS_KOP_TEST-yCkgCdqG");
			}

			@Test
			void shouldMapFormName() {
				var data = engineAdapter.parseFormData(formData);

				assertThat(data.getHeader().getFormName()).isEqualTo("KFAS_KOP_TEST");
			}

			@Test
			void shouldMapFormId() {
				var data = engineAdapter.parseFormData(formData);

				assertThat(data.getHeader().getFormId()).isEqualTo("KFAS_KOP_TEST");
			}
		}

		@Nested
		class TestPanels {

			@Test
			void shouldMap() {
				var data = engineAdapter.parseFormData(formData).getFormData();

				assertThat(data).isNotNull();
			}

			@Test
			void shouldMapIdentifier() {
				var data = engineAdapter.parseFormData(formData).getFormData();

				assertThat(data).containsKey("Panel_0");
			}

			@Test
			void shouldMapComponents() {
				var data = engineAdapter.parseFormData(formData).getFormData();

				assertThat(getPanel0(data)).isNotNull();
			}

			@Test
			void shouldMapComponentIdentifier() {
				var data = engineAdapter.parseFormData(formData).getFormData();

				assertThat(getPanel0(data)).containsKey("1208_Hund Grunddaten");
			}

			@Test
			@SuppressWarnings("unchecked")
			void shouldMapNestedComponents() {
				var data = engineAdapter.parseFormData(formData).getFormData();

				assertThat((Map<String, Object>) getPanel0(data).get("1208_Hund Grunddaten")).isNotNull();
			}

			@Test
			@SuppressWarnings("unchecked")
			void shouldMapNestedSubComponentIdentifier() {
				var data = engineAdapter.parseFormData(formData).getFormData();

				assertThat((Map<String, Object>) getPanel0(data).get("1208_Hund Grunddaten"))
						.containsKey("1208_Hund Grunddaten");
			}

			@Test
			@SuppressWarnings("unchecked")
			void shouldMapNestedSubComponents() {
				var data = engineAdapter.parseFormData(formData).getFormData();

				assertThat((Map<String, Object>) ((Map<String, Object>) (getPanel0(data)).get("1208_Hund Grunddaten"))
						.get("1208_Hund Grunddaten")).containsKey("Hund_Name");
			}

			@SuppressWarnings("unchecked")
			private Map<String, Object> getPanel0(Map<String, Object> data) {
				return ((Map<String, Object>) data.get("Panel_0"));
			}
		}

		@Nested
		class TestZutaendigeStelle {
			@Test
			void shouldMap() {
				var data = engineAdapter.parseFormData(formData);

				assertThat(data.getZustaendigeStelle()).isNotNull();
			}

			@Test
			void shouldMapOrganistaionseinheitenId() {
				var data = engineAdapter.parseFormData(formData);

				assertThat(data.getZustaendigeStelle().getOrganisationseinheitenId()).isEqualTo("test");
			}
		}
	}

	private FormData prepareTestData() {
		var fileGroup = IncomingFileGroupTestFactory.createBuilder()
				.name(FormSolutionsFilesMapper.FILE_GROUP_ZIP_NAME)
				.clearFiles()
				.file(IncomingFileTestFactory.createBuilder()
						.name(FILE_NAME_ZIP_ATTACHMENT)
						.file(asFile(FILE_NAME_ZIP_ATTACHMENT))
						.size(getFileSize(FILE_NAME_ZIP_ATTACHMENT))
						.contentType(ZIP_CONTENT_TYPE)
						.build())
				.build();

		return FormData.builder().formData(getTestDataFromFile()).attachment(fileGroup).build();
	}

	@SneakyThrows
	private long getFileSize(String fileName) {
		Path filePath = Path.of(FormSolutionsEngineBasedAdapterITCase.class.getClassLoader().getResource(fileName).toURI());
		return Files.size(filePath);
	}

	private Map<String, Object> getTestDataFromFile() {
		try {
			var testData = TestUtils.loadTextFile("formsolutions001.json");
			return objectMapper.readValue(testData, new TypeReference<>() {
			});
		} catch (JsonProcessingException e) {
			var msg = Optional.ofNullable(e.getCause()).map(Throwable::getMessage).orElseGet(e::getMessage);
			throw new TechnicalException("Error parsing test JSON " + msg, e);
		}
	}

	private File asFile(String path) {
		return TempFileUtils.writeTmpFile(TestUtils.loadFile(path));
	}
}
