/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.eingang.xta;

import java.util.Spliterators;
import java.util.stream.Stream;
import java.util.stream.StreamSupport;

import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import de.ozgcloud.eingang.common.formdata.FormData;
import de.ozgcloud.eingang.common.formdata.FormHeader;
import de.ozgcloud.eingang.common.vorgang.VorgangNummerSupplier;
import lombok.NonNull;
import lombok.extern.log4j.Log4j2;

@Service
@Log4j2
class XtaService {

	static final String DFOERDERMITTELANTRAG_MESSAGE_TYPE = "Geschaeftsgang.Geschaeftsgang.0201";
	static final int VORGANG_NUMMER_SUFFIX_LENGTH = 4;

	@Autowired
	private XtaRemoteService remoteService;
	@Autowired
	private XtaMessageMapper mapper;
	@Autowired
	private VorgangNummerSupplier vorgangNummerSupplier;

	public Stream<FormData> getMessages() {
		return createXtaMessageStream().filter(this::filterByMessageType).map(this::getFormData);
	}

	Stream<XtaMessageMetaData> createXtaMessageStream() {
		var iterator = new XtaMessageMetadataRemoteIterator(remoteService);
		return StreamSupport.stream(Spliterators.spliteratorUnknownSize(iterator, 0), false);
	}

	// filter criteria for dFoerdermittelantrag
	// https://jira.mgm-tp.com/jira/browse/OZG-3659
	boolean filterByMessageType(XtaMessageMetaData metaData) {
		if (StringUtils.equals(metaData.getMessageType(), DFOERDERMITTELANTRAG_MESSAGE_TYPE)) {
			return true;
		}

		LOG.warn("Ignoring XTA-Message of type '{}'.", metaData.getMessageType());
		return false;
	}

	public FormData getFormData(@NonNull XtaMessageMetaData metaData) {
		var msg = remoteService.getMessage(metaData.getMessageId());
		var formData = mapper.toFormData(msg.toBuilder().metaData(metaData).build());
		return updateHeader(formData);
	}

	FormData updateHeader(FormData formData) {
		return formData.toBuilder().header(setVorgangNummer(formData.getHeader())).build();
	}

	FormHeader setVorgangNummer(FormHeader formHeader) {
		return formHeader.toBuilder().vorgangNummer(vorgangNummerSupplier.get(VORGANG_NUMMER_SUFFIX_LENGTH)).build();
	}

	public void acknowledgeReceive(@NonNull XtaMessageId messageId) {
		remoteService.close(messageId);
	}
}
