/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.eingang.xta;

import static org.assertj.core.api.Assertions.*;
import static org.assertj.core.api.InstanceOfAssertFactories.*;
import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.mockito.ArgumentCaptor;
import org.mockito.Captor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.boot.test.mock.mockito.MockBean;
import org.springframework.context.annotation.Bean;
import org.springframework.test.context.ActiveProfiles;

import de.ozgcloud.common.test.TestUtils;
import de.ozgcloud.eingang.Application;
import de.ozgcloud.eingang.common.formdata.FormData;
import de.ozgcloud.eingang.common.formdata.ServiceKonto;
import de.ozgcloud.eingang.common.formdata.ServiceKonto.PostfachAddress;
import de.ozgcloud.eingang.common.formdata.StringBasedIdentifier;
import de.ozgcloud.eingang.router.VorgangRemoteService;

@SpringBootTest(classes = { Application.class, ActivateXTARunnerConfig.class })
@ActiveProfiles({ "local", "itcase" })
class XtaITCase {

	@MockBean
	private XtaRemoteService remoteService;
	@MockBean
	private VorgangRemoteService vorgangRemoteService;

	@Autowired
	private XtaRunner runner;
	@Captor
	private ArgumentCaptor<FormData> formDataCaptor;

	@BeforeEach
	void prepareXtaRemoteService() {
		when(remoteService.getMessagesMetadata()).thenReturn(XtaMessageMetaDatasAndHeaderTestFactory.create());

		when(remoteService.getMessage(any(XtaMessageId.class))).thenReturn(
				XtaMessageTestFactory.createBuilder()
						.clearMessageFiles()
						.messageFile(XtaFileTestFactory.withFileContent(TestUtils.loadFile("test_content.zip"), "test_content.zip"))
						.build());
	}

	@Test
	void shouldGetMessage() {
		runner.onApplicationEvent(null);

		verify(remoteService).getMessage(XtaMessageTestFactory.MESSAGE_ID);
	}

	@Test
	void shouldCloseMessage() {
		runner.onApplicationEvent(null);

		verify(remoteService, timeout(2000)).close(XtaMessageTestFactory.MESSAGE_ID);
	}

	@Test
	void shouldSend4Representations() {
		runner.onApplicationEvent(null);

		verify(vorgangRemoteService).createVorgang(formDataCaptor.capture(), any(), any());
		assertThat(formDataCaptor.getValue().getNumberOfRepresentations()).isEqualTo(4);
		assertThat(formDataCaptor.getValue().getRepresentations()).hasSize(4);
	}

	@Test
	void shouldHavePostfachId() {
		runner.onApplicationEvent(null);

		var formData = captorFormData();

		assertThat(formData.getHeader().getServiceKonto()).describedAs("ServiceKonto").isNotNull()
				.extracting(ServiceKonto::getPostfachAddresses).asList().hasSize(1).first()
				.asInstanceOf(type(PostfachAddress.class))
				.extracting(PostfachAddress::getIdentifier).isInstanceOf(StringBasedIdentifier.class).asInstanceOf(type(StringBasedIdentifier.class))
				.extracting(StringBasedIdentifier::getPostfachId).isEqualTo("4dd01647-b9d9-4775-1b50-08da3d83800a");
	}

	@Test
	void shouldHaveOrganisationsEinheitId() {
		runner.onApplicationEvent(null);

		var formData = captorFormData();

		assertThat(formData.getZustaendigeStelle().getOrganisationseinheitenId()).isEqualTo("9795669");
	}

	@Test
	void shouldHaveVorgangNummer() {
		runner.onApplicationEvent(null);

		var formData = captorFormData();

		assertThat(formData.getHeader().getVorgangNummer()).hasSize(9);
	}

	private FormData captorFormData() {
		verify(vorgangRemoteService).createVorgang(formDataCaptor.capture(), any(), any());

		return formDataCaptor.getValue();
	}

}

class ActivateXTARunnerConfig {
	@Bean
	XtaRunner xtaRunner() {
		return new XtaRunner();
	}
}